import QUB
from Buttons import *
from qubtree import Node
from qubtree import QTR_TYPE_FLOAT
import math


def Derivate(lst, deltaP, pcter=lambda x:x):
    '''returns [ lst[i+deltaP] - lst[i] ]'''
    if len(lst) <= deltaP:
        return []
    der = []
    nn = 1.0 * len(lst)
    for i in range(len(lst) - deltaP):
        der.append( lst[i+deltaP] - lst[i] )
        if i%50 == 0:
            pcter(i/nn)
    return der

def FindPeaks(lst, minPeak, skip, pcter=lambda x:x):
    peaks = []
    if len(lst) <= 1:
        return peaks

    nn = len(lst)    
    if lst[0] > lst[1] and lst[0] >= minPeak:
        peaks.append( (0, lst[0]) )
    for i in range(1, len(lst)-1):
        if (not len(peaks)) or ((i - peaks[len(peaks)-1][0]) > skip):
            v = lst[i]
            if v > lst[i-1] and v >= lst[i+1] and v >= minPeak:
                peaks.append( (i, v) )
        if i%50 == 0:
            pcter(i/nn)
    i = len(lst) - 1
    if lst[i] > lst[i-1] and lst[i] >= minPeak:
        peaks.append( (i, lst[i]) )
    return peaks            
    

def FindNegPeaks(lst, minPeak, skip, pcter=lambda x:x):
    peaks = []
    if len(lst) <= 1:
        return peaks

    nn = len(lst)    
    if lst[0] < lst[1] and lst[0] <= minPeak:
        peaks.append( (0, lst[0]) )
    for i in range(1, len(lst)-1):
        if (not len(peaks)) or ((i - peaks[len(peaks)-1][0]) > skip):
            v = lst[i]
            if v < lst[i-1] and v <= lst[i+1] and v <= minPeak:
                peaks.append( (i, v) )
        if i%50 == 0:
            pcter(i/nn)
    i = len(lst) - 1
    if lst[i] < lst[i-1] and lst[i] <= minPeak:
        peaks.append( (i, lst[i]) )

    return peaks            
    

def AvgContext(seg, p0, deltaP, contextPreP, contextPostP):
    def Avg(ixs):
        sum, count = 0.0, 0
        for ix in ixs:
            sum += data[ix]
            count += 1
        if count > 0:
            sum /= count
        return sum
            
    data = seg['Channel'].data
    N = len(data)

    return (Avg(xrange(max(0, p0-contextPreP), max(0, p0))),
            Avg(xrange(min(N, p0+deltaP+1), min(N, p0+deltaP+contextPostP+1))))


def FindJumps(props):
    deltaT = props['deltaT']
    deltaY = props['deltaY']
    contextPreT = props['contextPre']
    contextPostT = props['contextPost']

    #----- Point XFindPeaks to positive or negative peaks function 
    XFindPeaks = FindPeaks
    if deltaY < 0:
        XFindPeaks = FindNegPeaks

    dsIn = QUB.getChosenData()[0]
    dsIn['ProcessData'].data = 'AsDisplayed'
    sampling = dsIn['sampling'].data[0]
    deltaP = int(round(deltaT / sampling))
    contextPreP = int(round(contextPreT / sampling))
    contextPostP = int(round(contextPostT / sampling))

    optionalContextP = 0
    if props['skipContext'] == 'yes':
        optionalContextP = contextPostP

    lastJumpOnly = props['findJumps'] == 'last only'        

    result = Node('ResultsFile')
    result['ResultType'].data = 'FindJumps'
    dsOut = result['DataSet']
    dsOut.appendClone(dsIn['FileName'])

    wiz = QUB.CreateWizard()
    wiz.caption = 'Finding Jumps...'
    wiz.show()
    status = ''
    def pcter(pct):
        wiz.message = '%s...%f%%' % (status, pct*100)

    # count segments:
    nseg = 0
    segIn = dsIn.find('Segment')
    while not segIn.isNull:
        nseg = nseg + 1
        segIn = segIn.nextSameName()

    ss = 0
    subpcter = lambda pct: pcter((ss + pct) / nseg)
    derpcter = lambda pct: subpcter(pct/2.0)
    peakpcter = lambda pct: subpcter(0.5+pct/2.0)

    outlist = []    

    segIn = dsIn.find('Segment')
    while not segIn.isNull:
        QUB.fillData(segIn)
        status = 'Deriving'
        deriv = Derivate(segIn['Channel'].data, deltaP, derpcter)
        #-----
        peaks = XFindPeaks(deriv, deltaY, deltaP + optionalContextP, peakpcter)
        
        #----- Strip last item only from list if lastJumpOnly is set 
        if lastJumpOnly:
            peaks = peaks[-1:]            

        wiz.message = 'Building list of %d peaks...' % len(peaks)
        for i, y in peaks:
            first = max(segIn.data[0], segIn.data[0] + i - contextPreP)
            last = min(segIn.data[1], segIn.data[0] + i + deltaP + contextPostP - 1)
            outlist.append( (first, last) )
            segOut = dsOut.append('Segment')
            segOut.data = (first, last)
            segOut['start'].data = segIn['start'].data[0] + sampling * (first - segIn.data[0])
            segOut['length'].data = (last - first + 1) * sampling
            segOut['Jump'].data = y
            segOut['PreAvg'].data, segOut['PostAvg'].data = AvgContext(segIn, i, deltaP, contextPreP, contextPostP)
            segOut['DeltaAvg'].data = segOut['PostAvg'].data[0] - segOut['PreAvg'].data[0]
        segIn.remove(segIn['Channel'])
        segIn = segIn.nextSameName()
        ss = ss + 1

    QUB.ResultsView.addResults(result)
    QUB.DataView.currData.setList(props['outname'], outlist)
    QUB.DataView.currData.chooseList(props['outname'])
    
    # get chosen data
    # convert time into points
    # setup result dataset
    # foreach input segment
    #   load data
    #   generate derivative
    #   list peaks greater than deltaY
    #   foreach peak:
    #     append output segment from peakX - context to peakX + deltaT + context
    #     with Jump = peakY


AddTaskButton( 'FindJumps', # button text
               FindJumps,   # function
               'Jumps',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Find Jumps",   # tool-tip
               "Ask Arthur Beyder",   # description in "Edit Buttons"
               "FindJumps Properties:", # caption for the settings dialog
               [DialogText("deltaT", "Delta t (ms):", 0.5, acceptFloatGreaterThan(0.0)),
                DialogText("deltaY", "Min delta y:", 1.0, acceptFloat),
                DialogText("contextPre", "Context pre (ms):", 0.5, acceptFloatGreaterThan(-0.00001)),
                DialogText("contextPost", "Context post (ms):", 0.5, acceptFloatGreaterThan(-0.00001)),
                DialogRadioGroup("skipContext", "Skip Context:", ['yes', 'no'], 'yes'),
                DialogRadioGroup("findJumps", "Find Jumps:", ['all', 'last only'], 'all'),
                DialogText("msecSkip", "Skip initial msec's:", 0.0, acceptFloatGreaterThan(-0.00001)),
                DialogText("outname", "Output list:", "Jumps", acceptString)
                ],
               'Modeling',
               DataChannel=True, DataList=True, Process=True)


def Min2Max(props):
    wiz = QUB.CreateWizard()
    wiz.caption = 'Finding Min2Max...'
    wiz.show()
    
    dsIn = QUB.getChosenData()[0]
    dsIn['ProcessData'].data = 'AsDisplayed'
    sampling = dsIn['sampling'].data[0]

    result = Node('ResultsFile')
    result['ResultType'].data = 'Min2Max'
    dsOut = result['DataSet']
    dsOut.appendClone(dsIn['FileName'])

    outlist = []    

   # count segments:
    nseg = 0
    segIn = dsIn.find('Segment')
    while not segIn.isNull:
        nseg = nseg + 1
        segIn = segIn.nextSameName()

    ss = 0
    segIn = dsIn.find('Segment')
    while not segIn.isNull:
        wiz.message = 'Selection %d/%d' % (ss+1, nseg)
        QUB.fillData(segIn)
        channel = segIn['Channel'].data
        QUB.Report(channel)
        minI, minY, maxI, maxY = 0, channel[0], 0, channel[0]
        for i in range(1, len(channel)):
            y = channel[i]
            if y < minY:
                minI, minY = i, y
            if y > maxY:
                maxI, maxY = i, y

        if minI < maxI:
            first, last = segIn.data[0] + minI, segIn.data[0] + maxI
            outlist.append( (first, last) )
            segOut = dsOut.append('Segment')
            segOut.data = (first, last)
            segOut['start'].data = segIn['start'].data[0] + sampling * minI
            segOut['length'].data = sampling * (maxI - minI + 1)
            segOut['Jump'].data = maxY - minY
        
        segIn.remove( channel.node )
        segIn = segIn.nextSameName()
        ss = ss + 1

    QUB.ResultsView.addResults(result)
    QUB.DataView.currData.setList(props['outname'], outlist)
    QUB.DataView.currData.chooseList(props['outname'])
    


AddTaskButton( 'Min2Max', # button text
               Min2Max,   # function
               'Mn2Mx',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Min2Max",   # tool-tip
               "Ask Arthur Beyder",   # description in "Edit Buttons"
               "Min2Max Properties:", # caption for the settings dialog
               [DialogText("outname", "Output list:", "Jumps", acceptString)],
               'Modeling',
               DataChannel=True, DataList=True, Process=True)



